package Model;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import Annotation.DBAnnotation;
import DAL.*;

/**
 * The Class Person.
 */
public class Person {

	/** The id. */
	protected Integer id;

	/** The first name. */
	protected String firstName;

	/** The last name. */
	protected String lastName;

	/** The gender. */
	protected Character gender;

	/** The biography. */
	protected String biography;

	/** The movies as a castmember. */
	ArrayList<Movie> moviesAsACastmember;

	/** The movies as a director. */
	ArrayList<Movie> moviesAsADirector;

	/** The movies as a producer. */
	ArrayList<Movie> moviesAsAProducer;

	/** The movies as a writer. */
	ArrayList<Movie> moviesAsAWriter;


	/**
	 * Instantiates a new person.
	 *
	 * @param id the id
	 * @param firstName the first name
	 * @param lastName the last name
	 * @param gender the gender
	 */
	public Person(Integer id, String firstName, String lastName,
			Character gender) {
		super();
		this.id = id;
		this.firstName = firstName;
		this.lastName = lastName;
		this.gender = gender;
	}

	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public Integer getId() {
		return id;
	}

	/**
	 * Sets the id.
	 *
	 * @param id the id to set
	 */
	public void setId(Integer id) {
		this.id = id;
	}

	/**
	 * Gets the first name.
	 *
	 * @return the firstName
	 */
	public String getFirstName() {
		return firstName;
	}

	/**
	 * Sets the first name.
	 *
	 * @param firstName the firstName to set
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * Gets the last name.
	 *
	 * @return the lastName
	 */
	public String getLastName() {
		return lastName;
	}

	/**
	 * Sets the last name.
	 *
	 * @param lastName the lastName to set
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	/**
	 * Gets the gender.
	 *
	 * @return the gender
	 */
	public Character getGender() {
		return gender;
	}

	/**
	 * Sets the gender.
	 *
	 * @param gender the gender to set
	 */
	public void setGender(Character gender) {
		this.gender = gender;
	}

	/**
	 * Gets the biography.
	 *
	 * @return the biography
	 */
	public String getBiography() {
		if (biography == null)
			biography = DAL.get().getPersonBiography(this.id);

		return biography;
	}

	/**
	 * Sets the biography.
	 *
	 * @param biography data
	 */
	public void setBiography(String biography) {
		this.biography = biography; 
	}

	/**
	 * Gets the movies as a castmember.
	 *
	 * @return the movies as a castmember
	 */
	public ArrayList<Movie> getMoviesAsACastmember() {
		if ( this.moviesAsACastmember == null)
		{
			ArrayList <Movie> movies = DAL.get().getMoviesFromCastmember(this.id);
			return movies;
		}
		return this.moviesAsACastmember;
	}

	/**
	 * Sets the movies as a castmember.
	 *
	 * @param movies the new movies as a castmember
	 */
	public void setMoviesAsACastmember(ArrayList<Movie> movies) {
		this.moviesAsACastmember = movies;
	}

	/**
	 * Gets the movies as a director.
	 *
	 * @return the movies as a director
	 */
	public ArrayList<Movie> getMoviesAsADirector() {
		if ( this.moviesAsADirector == null )
		{
			ArrayList <Movie> movies = DAL.get().getMoviesFromDirector(this.id);
			return movies;
		}
		return this.moviesAsADirector;
	}

	/**
	 * Sets the movies as a director.
	 *
	 * @param movies the new movies as a director
	 */
	public void setMoviesAsADirector(ArrayList<Movie> movies) {
		this.moviesAsADirector = movies;
	}

	/**
	 * Gets the movies as a producer.
	 *
	 * @return the movies as a producer
	 */
	public ArrayList<Movie> getMoviesAsAProducer() {
		if ( this.moviesAsAProducer == null )
		{
			ArrayList <Movie> movies = DAL.get().getMoviesFromProducer(this.id);
			return movies;
		}
		return this.moviesAsAProducer;
	}

	/**
	 * Sets the movies as a producer.
	 *
	 * @param movies the new movies as a producer
	 */
	public void setMoviesAsAProducer(ArrayList<Movie> movies) {
		this.moviesAsAProducer = movies;
	}

	/**
	 * Gets the movies as a writer.
	 *
	 * @return the movies as a writer
	 */
	public ArrayList<Movie> getMoviesAsAWriter() {
		if ( this.moviesAsAWriter == null )
		{
			ArrayList <Movie> movies = DAL.get().getMoviesFromWriter(this.id);
			return movies;
		}
		return this.moviesAsAWriter;
	}

	/**
	 * Sets the movies as a writer.
	 *
	 * @param movies the new movies as a writer
	 */
	public void setMoviesAsAWriter(ArrayList<Movie> movies) {
		this.moviesAsAWriter = movies;
	}

	public void updateStore_IDToOne(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE, SSN FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			int addressid = rs.getInt("ADDRESS_ID");	
			DBAnnotation.annoate("addressid","MOCK_DATA5000","ADDRESS_ID",true);
			assert(addressid == 2);

			int storeid = 2; // should be 1
			DBAnnotation.annoate("storeid","MOCK_DATA5000","STORE_ID",false);



			String updateS = "UPDATE PERSON SET STORE_ID = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, storeid);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();
			System.out.println("Done!");
		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}


	public void updateStore_IDToOne2Correct(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE, SSN FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			int addressid = rs.getInt("ADDRESS_ID");	
			DBAnnotation.annoate("addressid","MOCK_DATA5000","ADDRESS_ID",true);
			assert(addressid == 2);

			int storeid = 1;
			DBAnnotation.annoate("storeid","MOCK_DATA5000","STORE_ID",false);



			String updateS = "UPDATE PERSON SET STORE_ID = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, storeid);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();
			System.out.println("Done!");
		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}






	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Person other = (Person) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}


	public void updateTimeSLOTLS35(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","AGE",true);

			int timeslot;

			double  bonus = 0;
			if(age < 35)
			{
				//				bonus = salary - 500;
				//				DBAnnotation.annoate("bonus","PERSON","BONUS",false);
				timeslot = 1;
				DBAnnotation.annoate("timeslot","MOCK_DATA5000","VISIT_TIMESLOT",false);
			}
			//else
			//			{
			//				timeslot = 4;
			//				//DBAnnotation.annoate("house","PERSON","HOUSE",false);
			//			}

			String updateS = "UPDATE PERSON SET BONUS = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, bonus);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}


	public void updateTimeSLOTGT35(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);


			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","AGE",true);

			int timeslot;

			if(age > 55)
			{
				//				bonus = salary - 500;
				//				DBAnnotation.annoate("bonus","PERSON","BONUS",false);
				timeslot = 4;   //should be 5
				DBAnnotation.annoate("timeslot","MOCK_DATA5000","VISIT_TIMESLOT",false);
			}
			else
			{
				timeslot = 1;
				DBAnnotation.annoate("timeslot","MOCK_DATA5000","VISIT_TIMESLOT",false);
				//DBAnnotation.annoate("house","PERSON","HOUSE",false);
			}

			String updateS = "UPDATE MOCK_DATA5000 SET VISIT_TIMESLOT = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, timeslot);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();
			System.out.println("Done!");
		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}


	public void TimeSlotToTwo(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){


			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","AGE",true);

			int timeslot;
			assert(age < 35);

			timeslot = 2;   //correct
			DBAnnotation.annoate("timeslot","MOCK_DATA5000","VISIT_TIMESLOT",false);

			String updateS = "UPDATE MOCK_DATA5000 SET VISIT_TIMESLOT = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, timeslot);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();
		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}

	public void updateMarriedLS35(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select married FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","AGE",true);

			int married = 0;

			if(age < 35)
			{

				married = 1;   //wrong , should be 0
				DBAnnotation.annoate("married","MOCK_DATA5000","married",false);
			}
			//else
			//			{
			//				timeslot = 4;
			//				//DBAnnotation.annoate("house","PERSON","HOUSE",false);
			//			}

			String updateS = "UPDATE PERSON SET married = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, married);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}


	public void updateMarriedGT35(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);


			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","AGE",true);

			int married =1;

			if(age > 35)
			{
				//				bonus = salary - 500;
				//				DBAnnotation.annoate("bonus","PERSON","BONUS",false);
				married = 0;   //should be 1
				DBAnnotation.annoate("married","MOCK_DATA5000","married",false);
			}

			String updateS = "UPDATE MOCK_DATA5000 SET married = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, married);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();
			System.out.println("Done!");
		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}


	public void UpdateMarriageToOne(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){


			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","AGE",true);

			assert(age > 25);

			int married =1;  //correct 
			DBAnnotation.annoate("married","MOCK_DATA5000","married",false);
			String updateS = "UPDATE MOCK_DATA5000 SET married = ?  WHERE SSN = ?";


			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, married);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();
		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}



	public void updateVisitTimeLS45(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","age",true);

			int visit_times_week = 0;

			if(age < 45)
			{

				visit_times_week = 3;   //wrong , should be 4
				DBAnnotation.annoate("visit_times_week","MOCK_DATA5000","visit_times_week",false);
			}
			//else
			//			{
			//				timeslot = 4;
			//				//DBAnnotation.annoate("house","PERSON","HOUSE",false);
			//			}

			String updateS = "UPDATE PERSON SET visit_times_week = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, visit_times_week);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}



	public void updateVisitTimeGT45(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","age",true);

			int visit_times_week = 0;

			if(age > 45)
			{

				visit_times_week = 1;   //wrong , should be 2
				DBAnnotation.annoate("visit_times_week","MOCK_DATA5000","visit_times_week",false);
			}


			String updateS = "UPDATE PERSON SET visit_times_week = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, visit_times_week);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}


	public void updateVisitTimeTo2(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","age",true);

			int visit_times_week = 0;

			assert(age > 45);
			visit_times_week = 2;   // 2, correct 
			DBAnnotation.annoate("visit_times_week","MOCK_DATA5000","visit_times_week",false);


			String updateS = "UPDATE PERSON SET visit_times_week = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, visit_times_week);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}

	
	public void updateNumberAccountLT35(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","age",true);

			int num_account = 0;

			if(age < 35)
			{

				num_account = 2;   //wrong , should be 3
				DBAnnotation.annoate("num_account","MOCK_DATA5000","num_account",false);
			}


			String updateS = "UPDATE PERSON SET visit_times_week = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, num_account);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}
	
	
	public void updateNumberAccountGT35(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","age",true);

			int num_account = 0;

			if(age > 35)
			{

				num_account = 1;   //wrong , should be 5
				DBAnnotation.annoate("num_account","MOCK_DATA5000","num_account",false);
			}


			String updateS = "UPDATE PERSON SET num_account = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, num_account);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}
	

	

	public void updateNumberAccountTo2(String ssn){
		try {

			Connection conn = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/simpletest",
					"root", "boyang");

			Statement stmt = (Statement) conn.createStatement();
			String query = "select AGE FROM mock_data where ssn = " + ssn;	
			ResultSet rs = stmt.executeQuery(query);

			//while(rs.next()){



			double age = rs.getInt("AGE");
			DBAnnotation.annoate("age","MOCK_DATA5000","age",true);

			int num_account = 0;

			assert(age > 45);
			num_account = 2;   // 2, correct 
			DBAnnotation.annoate("num_account","MOCK_DATA5000","num_account",false);


			String updateS = "UPDATE PERSON SET num_account = ?  WHERE SSN = ?";

			PreparedStatement preparedStmt =  (PreparedStatement) conn.prepareStatement(updateS);
			preparedStmt.setDouble(1, num_account);
			preparedStmt.setString(2, ssn);

			preparedStmt.executeUpdate();
			//DBAnnotation.annoate("bonus","PERSON","BONUS",false);

			conn.close();

		}
		catch (Exception e) {
			System.out.println("exception");
		}
	}

	
	
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "Person \n{\nid=" + id + ", \nfirstName=" + firstName
				+ ", \nlastName=" + lastName + ", \ngender=" + gender
				+ ", \nbiography=" + biography + "\n}";
	}
}
